==> pwa_firebase-2.0.5.tar.gz: pwa_firebase/src/Firebase.php <==
<?php

namespace Drupal\pwa_firebase;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\ClientInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Service to send notification.
 */
class Firebase implements FirebaseInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * Constructs a FirebaseServiceBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The factory for configuration objects.
   * @param \GuzzleHttp\ClientInterface $client
   *   An HTTP client.
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   */
  public function __construct(ConfigFactoryInterface $configFactory, ClientInterface $client, Connection $connection, MessengerInterface $messenger, ModuleHandlerInterface $module_handler) {
    $this->configFactory = $configFactory;
    $this->client = $client;
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('http_client'),
      $container->get('database'),
      $container->get('messenger'),
      $container->get('module_handler')
    );
  }

  /**
   * Function to send a notification to all the users.
   */
  public function sendMessageToAllUsers($title = '', $message = '', $url = NULL, $option = []) {

    $rows = $this->connection->select('pwa_firebase', 'f')
      ->fields('f', ['token', 'device'])
      ->execute()
      ->fetchAll();
    $this->prepareSend($rows, $title, $message, $url, $option);
  }

  /**
   * Function to send a notification to 1 user.
   */
  public function sendMessageToUser($uid, $title, $message, $url = NULL, $option = []) {

    $rows = $this->connection->select('pwa_firebase', 'f')
      ->fields('f', ['token', 'device'])
      ->condition('uid', $uid)
      ->execute()
      ->fetchAll();
    $this->prepareSend($rows, $title, $message, $url, $option);
  }

  /**
   * {@inheritDoc}
   */
  protected function prepareSend($rows, $title, $message, $url = NULL, $option = []) {
    $tokens = [];
    foreach ($rows as $row) {
      $token = $row->token;
      $device_type = 'pc';
      if (!empty($row->device)) {
        $device_type = $row->device;
      }
      if (!isset($tokens[$device_type])) {
        $tokens[$device_type] = [];
      }
      $tokens[$device_type][] = $token;
    }
    if (!empty($tokens)) {
      foreach ($tokens as $device => $token) {
        $option['device_type'] = $device;
        $this->sendNotification($token, $title, $message, $url, $option);
      }
    }
  }

  /**
   * Function send message to firebase.
   */
  public function sendNotification($token, $title, $message, $url = NULL, $option = []) {
    global $base_url;
    global $base_path;

    $config = $this->configFactory->get('pwa_firebase.settings');
    if (strlen($title) > 255) {
      $title = substr($title, 0, 254) . '…';
    }
    if (strlen($message) > 1024) {
      $message = substr($message, 0, 1023) . '…';
    }
    if (empty($url)) {
      $url = $base_url . $base_path;
    }

    $headers = [
      'Content-Type' => 'application/json',
      'Accept' => 'application/json',
      'Authorization' => 'key=' . $config->get('firebase_server_key'),
    ];

    $logo = theme_get_setting('logo.url');
    // Note: this is the image that is used for the manifest file.
    $icon = !empty($option['image']) ? $base_url . $base_path . $option['image'] : $logo;
    if (filter_var($icon, FILTER_VALIDATE_URL) == FALSE) {
      $icon = $base_url . $icon;
    }
    $this->moduleHandler->alter('pwa_firebase_send', $option);
    $notification = [
      'body' => $message,
      'title' => $title,
      'icon' => $icon,
      'click_action' => $url,
    ];
    foreach ($notification as $key => $value) {
      if (!empty($option[$key])) {
        $notification[$key] = $option[$key];
        unset($option[$key]);
      }
    }
    $data = [
      "content_available" => !empty($option['content_available']) == 1 ? FALSE : TRUE,
      "priority" => !empty($option['priority']) ? $option['priority'] : 'high',
      "notification" => $notification,
      "data" => $option,
    ];
    if (is_array($token)) {
      if (count($token) == 1) {
        $data["to"] = current($token);
      }
      else {
        $data["registration_ids"] = $token;
      }
    }
    else {
      $data["to"] = $token;
    }
    $result = FALSE;
    try {
      $response = $this->client->post($config->get('firebase_endpoint'), [
        'headers' => $headers,
        'body' => Json::encode($data),
      ]);
      $result = Json::decode($response->getBody());
    }
    catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
    }

    if (!empty($result["failure"])) {
      foreach ($result["results"] as $value) {
        if (!empty($value['error'])) {
          $this->messenger->addError($value['error']);
          // @todo remove token invalidate.
          /* $this->connection->delete('pwa_firebase')
          ->condition('token', $token[$index])
          ->execute();*/
        }
      }
    }
    return $result;
  }

}

==> pwa_firebase-2.0.7.tar.gz: pwa_firebase/src/Firebase.php <==
<?php

namespace Drupal\pwa_firebase;

use Drupal\Component\Serialization\Json;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Messenger\MessengerInterface;
use GuzzleHttp\ClientInterface;

/**
 * Service to send notification.
 */
class Firebase implements FirebaseInterface {

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactory
   */
  protected $configFactory;

  /**
   * HTTP client.
   *
   * @var \GuzzleHttp\Client
   */
  protected $client;

  /**
   * Database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  private $connection;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The entity services.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * Constructs a FirebaseServiceBase object.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The factory for configuration objects.
   * @param \GuzzleHttp\ClientInterface $client
   *   An HTTP client.
   * @param \Drupal\Core\Database\Connection $connection
   *   Database connection.
   * @param \Drupal\Core\Messenger\MessengerInterface $messenger
   *   The messenger service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler service.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity service.
   */
  public function __construct(ConfigFactoryInterface $config_factory, ClientInterface $client, Connection $connection, MessengerInterface $messenger, ModuleHandlerInterface $module_handler, EntityTypeManagerInterface $entity_type_manager) {
    $this->configFactory = $config_factory;
    $this->client = $client;
    $this->connection = $connection;
    $this->messenger = $messenger;
    $this->moduleHandler = $module_handler;
    $this->entityTypeManager = $entity_type_manager;
  }

  /**
   * Function to send a notification to all the users.
   *
   * @param string $title
   *   Message title.
   * @param string $message
   *   Message body.
   * @param mixed $url
   *   Url to article.
   * @param mixed $option
   *   Option for device type.
   */
  public function sendMessageToAllUsers($title = '', $message = '', $url = NULL, $option = []) {

    $rows = $this->connection->select('pwa_firebase', 'f')
      ->fields('f', ['token', 'device'])
      ->execute()
      ->fetchAll();
    return $this->prepareSend($rows, $title, $message, $url, $option);
  }

  /**
   * Function to send a notification to 1 user.
   */
  public function sendMessageToUser($uid, $title, $message, $url = NULL, $option = []) {

    $rows = $this->connection->select('pwa_firebase', 'f')
      ->fields('f', ['token', 'device'])
      ->condition('uid', $uid)
      ->execute()
      ->fetchAll();
    if (!empty($rows)) {
      $option['uid'] = $uid;
      $option['user_name'] = $this->entityTypeManager->getStorage('user')->load($uid)->getDisplayName();
      return $this->prepareSend($rows, $title, $message, $url, $option);
    }
    return FALSE;
  }

  /**
   * {@inheritDoc}
   */
  protected function prepareSend($rows, $title, $message, $url = NULL, $option = []) {
    $tokens = [];
    foreach ($rows as $row) {
      $token = $row->token;
      $device_type = 'pc';
      if (!empty($row->device)) {
        $device_type = $row->device;
      }
      if (!isset($tokens[$device_type])) {
        $tokens[$device_type] = [];
      }
      $tokens[$device_type][] = $token;
    }
    if (!empty($tokens)) {
      foreach ($tokens as $device => $token) {
        $option['device_type'] = $device;
        $this->sendNotification($token, $title, $message, $url, $option);
      }
      return TRUE;
    }
    return FALSE;
  }

  /**
   * Function send message to firebase.
   */
  public function sendNotification($token, $title, $message, $url = NULL, $option = []) {
    global $base_url;
    global $base_path;

    $config = $this->configFactory->get('pwa_firebase.settings');
    if (strlen($title) > 255) {
      $title = substr($title, 0, 254) . '…';
    }
    if (strlen($message) > 1024) {
      $message = substr($message, 0, 1023) . '…';
    }
    if (empty($url)) {
      $url = $base_url . $base_path;
    }

    $headers = [
      'Content-Type' => 'application/json',
      'Accept' => 'application/json',
      'Authorization' => 'key=' . $config->get('firebase_server_key'),
    ];

    $logo = theme_get_setting('logo.url');
    // Note: this is the image that is used for the manifest file.
    $icon = !empty($option['image']) ? $base_url . $base_path . $option['image'] : $logo;
    if (filter_var($icon, FILTER_VALIDATE_URL) == FALSE) {
      $icon = $base_url . $icon;
    }
    $this->moduleHandler->alter('pwa_firebase_send', $option);
    $notification = [
      'body' => $message,
      'title' => $title,
      'icon' => $icon,
      'click_action' => $url,
    ];
    foreach ($notification as $key => $value) {
      if (!empty($option[$key])) {
        $notification[$key] = $option[$key];
        unset($option[$key]);
      }
    }
    $data = [
      "content_available" => !empty($option['content_available']) == 1 ? FALSE : TRUE,
      "priority" => !empty($option['priority']) ? $option['priority'] : 'high',
      "notification" => $notification,
      "data" => $option,
    ];
    if (is_array($token)) {
      if (count($token) == 1) {
        $data["to"] = current($token);
      }
      else {
        $data["registration_ids"] = $token;
      }
    }
    else {
      $data["to"] = $token;
    }
    $result = FALSE;
    try {
      $response = $this->client->post($config->get('firebase_endpoint'), [
        'headers' => $headers,
        'body' => Json::encode($data),
      ]);
      $result = Json::decode($response->getBody());
    }
    catch (\Exception $e) {
      $this->messenger->addError($e->getMessage());
    }

    if (!empty($result["failure"])) {
      foreach ($result["results"] as $index => $value) {
        if (!empty($value['error'])) {
          $this->messenger->addError($value['error'] . ': ' . $option['user_name'] . ' - ' . $option['device']);
          $this->connection->delete('pwa_firebase')->condition('token', $token[$index])->execute();
        }
      }
    }
    return $result;
  }

}
